import sys

from memgen.parse_cli import parse_cli
from memgen.memgen import memgen


def main():
  args = parse_cli(sys.argv)

  memgen(args.input_pdbs, args.output_pdb, png=args.png, ratio=args.ratio,
      area_per_lipid=args.area_per_lipid,
      water_per_lipid=args.water_per_lipid,
      lipids_per_monolayer=args.lipids_per_monolayer,
      server=args.server
  )



import argparse

from memgen.box_shape import BoxShape
from memgen.defaults import Defaults


def parse_cli(argv):
    parser = argparse.ArgumentParser('memgen', formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("input_pdbs", nargs="+", help="PDB files with lipids.")
    parser.add_argument("output_pdb", help="Generated membrane.")

    parser.add_argument("-c", "--ratio", nargs="*", type=int,
            help=""" Lipid concentration ratio. For example: 1 4 (same as 20 80).
                     It means 20%% of the first lipid and 80%% of the second. """)
    parser.add_argument("-a", "--area-per-lipid", type=int, default=Defaults.area_per_lipid, help="Area per lipid (Å²)")
    parser.add_argument("-w", "--water-per-lipid", type=int, default=Defaults.water_per_lipid, help="Water molecules per lipid")
    parser.add_argument("-n", "--lipids-per-monolayer", type=int, default=Defaults.lipids_per_monolayer, help="Lipids per monolayer")
    parser.add_argument("-s", "--added-salt", type=int, default=0, help="Added salt (milli molar)")
    parser.add_argument("-b", "--box-shape", type=BoxShape, choices=list(BoxShape), default=BoxShape.square, help="Box shape")
    parser.add_argument("--png", help="A small thumbnail depicting generated membrane.")

    maintenance = parser.add_argument_group('Maintenance')
    maintenance.add_argument("--server", default=Defaults.server, help="Hostname of MemGen REST API server.")

    args = parser.parse_args(argv[1:])

    # Setting equal ratio of lipids if ratio not specified. For two lipids 1:1, for three 1:1:1 (33%/33%/33%) and so on.
    if len(args.input_pdbs) > 1 and args.ratio is None:
        args.ratio = [1] * len(args.input_pdbs)

    return args



import os.path
from tempfile import TemporaryDirectory

from memgen import memgen


def test_memgen():
  with TemporaryDirectory() as temporary_dir:
    output_pdb = f"{temporary_dir}/membrane.pdb"
    output_png = f"{temporary_dir}/membrane.png"

    memgen(["example/dmpc.pdb", "example/dopc.pdb"], output_pdb, png=output_png,
        ratio=[1, 4], area_per_lipid=65, water_per_lipid=40, lipids_per_monolayer=128)

    assert(os.path.exists(output_pdb))
    assert(os.path.exists(output_png))



# -*- coding: utf-8 -*-
import re
from setuptools import setup, find_packages

project_url = "https://gitlab.com/cbjh/memgen/py-memgen"

with open("README.md", "r", encoding="utf-8") as fp:
    readme_content = fp.read()
    fixed_images = re.sub(r"!\[([^\]]*)\]\(([^)]*)\)", r"![\1]({}/-/raw/main/\2)".format(project_url), readme_content)
    fixed_images_links = re.sub(r"\[([^\]]*)\]\(([^)]*)\)", r"[\1]({}/-/blob/main/\2)".format(project_url), fixed_images)

setup(
    name="memgen",
    author="Maciej Wójcik",
    author_email="w8jcik@gmail.com",
    description="Calls MemGen service located at http://memgen.uni-saarland.de/api",
    long_description=fixed_images_links,
    long_description_content_type="text/markdown",
    url=project_url,
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
    ],
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    install_requires=[],
    entry_points={
        'console_scripts': [
            'memgen=memgen:main',
        ]
    }
)



import base64
import http.client
import json
from typing import List, Optional

from memgen.parse_cli import BoxShape
from memgen.defaults import Defaults


def jupyter_display(png):
  if png is None:
    return None

  try:
    from IPython.display import Image

    return Image(filename=png)
  except ImportError as e:
    pass


def memgen(
    input_pdbs: List[str],
    output_pdb: str,
    ratio: Optional[List[int]] = None,
    area_per_lipid: int = Defaults.area_per_lipid,
    water_per_lipid: int = Defaults.water_per_lipid,
    lipids_per_monolayer: int = Defaults.lipids_per_monolayer,
    box_shape: BoxShape = Defaults.box_shape,
    png: Optional[str] = None,
    server: str = Defaults.server
):
  print(f'Submitting "{", ".join(input_pdbs)}" to "{server}". Please wait...')

  encoded_input_pdbs_content = []

  for input_pdb in input_pdbs:
    with open(input_pdb, 'rb') as fp:
      encoded_input_pdbs_content.append(str(base64.b64encode(fp.read()), 'ascii'))

  connection = http.client.HTTPConnection(server)

  connection.request('POST', '/api/submit', json.dumps({
    'pdbs': encoded_input_pdbs_content,
    'ratio': ratio,
    'areaPerLipid': area_per_lipid,
    'waterPerLipid': water_per_lipid,
    'lipidsPerMonolayer': lipids_per_monolayer
  }), {'Content-type': 'application/json'})

  res = connection.getresponse()

  res_body = json.loads(res.read().decode())

  if res.status == 200:
    with open(output_pdb, 'wb') as pdb_fp:
      pdb_fp.write(base64.b64decode(res_body['pdb']))

    if png:
      with open(png, 'wb') as png_fp:
        png_fp.write(base64.b64decode(res_body['png']))

    if png:
      print(f'Output saved as "{output_pdb}" and "{png}".')
    else:
      print(f'Output saved as "{output_pdb}".')

    return jupyter_display(png)
  else:
    print(f"Server response: {res_body['error']}")
    print(f'Saving error output as "{output_pdb}-out.log" and "{output_pdb}-out.log".')

    with open(f'{output_pdb}-out.log', 'w') as std_out_fp:
      std_out_fp.write(res_body['stdOut'])

    with open(f'{output_pdb}-err.log', 'w') as std_err_fp:
      std_err_fp.write(res_body['stdErr'])

    exit(1)



from memgen.parse_cli import parse_cli, BoxShape


def test_minimal_example():
    """ Minimal example. Single lipid, with default settings. """

    cmd = 'memgen example/dmpc.pdb membrane.pdb'

    assert vars(parse_cli(cmd.split())) == {
      'input_pdbs': ['example/dmpc.pdb'],
      'output_pdb': 'membrane.pdb',
      'ratio': None,
      'area_per_lipid': 65,
      'water_per_lipid': 35,
      'lipids_per_monolayer': 64,
      'added_salt': 0,
      'box_shape': BoxShape.square,
      'png': None,
      'server': 'memgen.uni-saarland.de'
    }


def test_kitchen_sink_example():
    """ Two lipids and all possible options set. """

    cmd = """ memgen 
              example/dmpc.pdb example/dopc.pdb membrane.pdb --ratio 1 4
              --area-per-lipid 55 --water-per-lipid 30 --lipids-per-monolayer 75 --added-salt 5
              --box-shape hexagonal
              --png membrane.png
              --server localhost:3000 """

    assert vars(parse_cli(cmd.split())) == {
      'input_pdbs': ['example/dmpc.pdb', 'example/dopc.pdb'],
      'output_pdb': 'membrane.pdb',
      'ratio': [1, 4],
      'area_per_lipid': 55,
      'water_per_lipid': 30,
      'lipids_per_monolayer': 75,
      'added_salt': 5,
      'box_shape': BoxShape.hexagonal,
      'png': 'membrane.png',
      'server': 'localhost:3000'
    }



from memgen.parse_cli import BoxShape


class Defaults:
  area_per_lipid: int = 65
  water_per_lipid: int = 35
  lipids_per_monolayer: int = 64
  box_shape = BoxShape.square

  server = "memgen.uni-saarland.de"
